/******************************************************************************
File:    S_TYPES.H
Project: SSCI
Purpose: Fundemental types and descriptors for SYSP (system processor).
Company: Comtrol Corporation
Author:  Jerry Medin
*******************************************************************************
Copyright 1993 Comtrol Corporation. All rights reserved.
Subject to developers license agreement.
*******************************************************************************
Comments: This file contains all defined types which are used by both SYSP and
          CTLP.  There is a companion file for the CTLP, called C_TYPES.H.
          Together these files allow the defined types to have
          the same sizes when using different compilers.  All types
          which must be shared between the SYSP and the CTLP must be
          defined in both C_TYPES.H and S_TYPES.H.  All defined types end
          in _T.

          When porting to a new memory model or new OS on either porcessor,
          the files C_TYPES.H and S_TYPES.H will need to be examined to ensure
          that all defined types have the same sizes on both SYSP and CTLP.
******************************************************************************/

#ifndef S_TYPES
#define S_TYPES 1

/******************************************************************************
                  Fundemental Types Common to SYSP and CTLP
******************************************************************************/
typedef signed char BYTE_T;            /* 8-bit signed */
typedef unsigned char UBYTE_T;         /* 8-bit unsigned  */
typedef signed short WORD_T;           /* 16-bit signed */
typedef unsigned short UWORD_T;        /* 16-bit unsigned */
typedef signed long LONG_T;            /* 32-bit signed */
typedef unsigned long ULONG_T;         /* 32-bit unsigned */
typedef unsigned char BOOL_T;          /* boolean */
#define FALSE     0                    /* values for BOOL_T */
#define TRUE      1
#define SUCCESS   1
#define FAIL      0
#define ON        1
#define OFF       0
#define FULL      1
#define NOTFULL   0


/******************************************************************************
                  Defined Types Unique to SYSP
******************************************************************************/
#include "c_8530.h"


/******************************************************************************
              Defined Types & Descriptors Common to SYSP and CTLP
******************************************************************************/
#include "sc_ip.h"


/******************************************************************************
                        Serial Line Descriptor
This LINE_T descriptor describes a serial line, which is a logical entity one
level above the hardware serial channel functions.  LINE_T descriptors are
accessible to both CTLP and SYSP, although some members may be used by
only one or the other.  Some of the members of LINE_T will have different
types on SYSP and CTLP.  These are all pointers, and can be identified by the
letter "P" at the end of the member name.
Naming convention:  "Ip" stands for "interprocessor."  "Loc" stands for
"local,"  "Local" refers to objects associated with a serial channel device.
"Interprocessor" refers to objects being passed across the interprocessor
boundary between the CTLP and the SYSP.  "CTLP" stands for "controller
processor," this is the on board microprocessor.  "SYSP" stands for "system
processor," this is the microprocessor in the host computer system.
******************************************************************************/
typedef struct
{
   WORD_T Num;                /* line number */
   WORD_T Dev;                /* device identifier (SYSP use) */
   UWORD_T Status;            /* status of the line (CTLP use) */
   UWORD_T SCPStatus;         /* SCP defined status word */
   UWORD_T Modes;             /* modes of operation */
   UWORD_T DetectEn;          /* ESC & SRC detection enable flags */
   UBYTE_T WR1State;          /* channel write register states */
   UBYTE_T WR3State;          
   UBYTE_T WR4State;
   UBYTE_T WR5State;
   UBYTE_T WR15State;
   UBYTE_T XOnChar;           /* XON character */
   UBYTE_T XOffChar;          /* XOFF character */
   UBYTE_T Reserve[11];       /* reserved for future use */
   UWORD_T ChanP;             /* ptr to channel descriptor (CTLP use) */
   UWORD_T SCPDescP;          /* ptr to SCP defined desc (CTLP use) */
   UWORD_T CtlP;              /* ptr to controller descriptor (SYSP use) */

   /* TxIpSDB.  This is the SDB where the SYSP is currently adding data. */
   WORD_T TxIpSDB;            /* current SDB handle */
   UBYTE_T far *TxIpBufP;     /* SYSP ptr to data buffer of current SDB */
   WORD_T TxIpWin;            /* SYSP DPM window where current SDB located */
   WORD_T TxIpCnt;            /* byte count in current SDB */

   /* TxIpQ.  This is a queues of transmit SDBs received from the SYSP. */
   WORD_T TxIpQ;              /* handle of 1st SDB on TxIpQ */
   WORD_T TxLocQ;             /* handle of 1st SDB on TxLocQ */
   WORD_T TxSDBCnt;           /* number of Tx SDBs on the line */
   WORD_T TxMaxSDB;           /* maximum number of SDBs allowed on this line */

   /* TxLocSDB.  Transmit data being removed from this SDB by the CTLP and
      written to the serial channel */
   WORD_T TxLocSDB;           /* handle of current SDB */
   ULONG_T TxLocBufP;         /* CTLP ptr to data buffer of current SDB */
   WORD_T TxLocCnt;           /* byte count in current buffer */
   BOOL_T TxImmedRdy;         /* a Tx immediate data byte is ready to send */
   UBYTE_T TxImmedData;       /* the Tx immediate data byte */

   /* RxIpSDB.  Receive data is being removed from this SDB by the SYSP. */
   WORD_T RxIpSDB;            /* handle of current SDB */
   UBYTE_T far *RxIpBufP;     /* SYSP ptr to data buffer of current SDB */
   WORD_T RxIpWin;            /* SYSP DPM window where current SDB located */
   WORD_T RxIpCnt;            /* byte count in current SDB */

   /* RxIpQ.  This is a queue of receive SDBs waiting to be read by the SYSP. */
   WORD_T RxIpQ;              /* handle of 1st SDB on RxIpQ */
   WORD_T RxLocQ;             /* handle of 1st SDB on local queue */
   WORD_T RxSDBCnt;           /* number of SDBs on both queues & bufs */
   WORD_T RxMaxSDB;           /* maximum number of SDBs allowed on this line */
   WORD_T HiWater;            /* num SDBs for flow control high water mark */
   WORD_T LoWater;            /* num SDBs for flow control low water mark */

   /* RxLocSDB.  Receive data read from the serial channel is being placed in
      this SDB by the CTLP. */
   WORD_T RxLocSDB;           /* handle of current SDB */
   ULONG_T RxLocBufP;         /* CTLP ptr to data buffer of current SDB */
   WORD_T RxLocCnt;           /* byte count in current buffer */
   WORD_T RxSDBTime;          /* controller time at which SDB put on RxLocQ */
} LINE_T;


/*-----------------------------------------------------------------------------
                        Fixed Dual Port Memory Descriptor
This describes all the descriptors that comprise the fixed DPM (dual
port memory) area except the line descriptor table and the DBCB_T table used
for SDBs.  The fixed DPM area is always located at the lowest
available DPM address.  The line table (type LINE_T) and SDB (shared data
buffer) free pool are also placed in the fixed DPM area, but
are not part of this structure because their sizes are determined when the
control program is compiled, not when the kernel library is compiled.  Since
C_TYPES.H is included in the kernel library source, it cannot have any
dependencies on defines that may be changed by the ComtOS application
programmer.

A file containing the address map corresponding to FIXDPM_T can be generated
with program FIXDPM.C.  If FIXDPM_T is changed FIXDPM.C must also be changed
or the address map will be incorrect.
-----------------------------------------------------------------------------*/
typedef struct
{
   FIRMUSER_T FirmUser;                /* firmware user area */
   TR_ENTRY_T TrBuf[TRBUF_SIZE];       /* trace buffer */
   WORD_T TrIx;                        /* trace buffer index */
   IPINT_T CtlInt;                     /* CTLP interprocessor interrupts */
   IPINT_T SysInt;                     /* SYSP interprocessor interrupts */
   UWORD_T StatusFlagGrp[SFG_SIZE];    /* interprocessor status flag groups */
   BOOL_T CtlDPAcc;                    /* CTLP dual port access request */
   BOOL_T SysDPAcc;                    /* SYSP dual port access request */
   ULONG_T SDBTblOff;                  /* offset in DPM of SDB table */
   WORD_T SDBFree;                     /* first SDB in free pool */
   WORD_T SDBFreeCnt;                  /* number of SDBs on free pool */
   UBYTE_T STKState;                   /* state of stack probes, defined below */
   UBYTE_T STKPrio;                    /* priority of task overflowing stk */
   UWORD_T STKPtr;                     /* overflowed stk ptr address */
   UWORD_T STKProbeP;                  /* lowest allowed stk ptr address */
   UBYTE_T Reserve[0x46];              /* reserved for future use */
   BYTE_T Dummy;                       /* used to get address of line table */
} FIXDPM_T;

#endif /* #ifndef S_TYPES */
